##############################################################################
#
# Copyright (c) 2002 Nik Kim <fafhrd@mindless.com> All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import Globals, traceback
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, ListFolderContents
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.Formulator.Errors import ValidationError, FormValidationError
from Group import Group
from CMFField import CMFField

factory_type_information = (
    {   'id'             : 'Form',
        'meta_type'      : 'CMF Form',
        'description'    : """Form""",
        'icon'           : 'Form.gif',
        'product'        : 'CMFFormulator',
        'factory'        : 'manage_addFormulatorForm',
        'immediate_view' : 'fform_edit_form',
        'filter_content_types' : 1,
        'allowed_content_types': ['Form IntegerField','Form StringField',
                                  'Form TextAreaField','Form EmailField',
                                  'Form CheckBoxField', 'Form RadioField',
                                  'Form MultiCheckBoxField',

                                  'MailStorage',
                                  ],
        'actions'        :
      ( { 'id'         : 'view',
          'name'       : 'View',
          'action'     : 'fform_view',
          'permissions': (View,),
          'category'   : 'object',
          },
        {'id'         : 'edit',
         'name'       : 'Edit',
         'action'     : 'fform_edit_form',
         'permissions': (ModifyPortalContent,),
         'category'   : 'object',
         },
        {'id'         : 'groups',
         'name'       : 'Groups',
         'action'     : 'fform_groups_form',
         'permissions': (ModifyPortalContent,),
         'category'   : 'object',
         },
        {'id'         : 'confirmation',
         'name'       : 'Confirmation',
         'action'     : 'confirmation',
         'permissions': (View,),
         'category'   : 'object',
         },
        {'id'         : 'metadata',
         'name'       : 'Metadata',
         'action'     : 'metadata_edit_form',
         'permissions': (ModifyPortalContent, ),
         'category'   : 'object',
          },

        {'id'         : 'folder_contents',
         'name'       : 'Folder contents',
         'action'     : 'folder_contents',
         'permissions': (ListFolderContents,),
         'category'      : 'folder',
         },
        
        )
        },
    )


class CMFForm(PortalFolder, PortalContent, DefaultDublinCoreImpl):
    ''' '''
    meta_type = 'CMF Form'
    portal_type = 'Form'

    __implements__ = ( PortalContent.__implements__,
                       DefaultDublinCoreImpl.__implements__ )

    security = ClassSecurityInfo()

    manage_options = (
        PortalFolder.manage_options[0],
        PortalContent.manage_options[0],) + PortalFolder.manage_options[1:]

    name = ''
    action = ''
    method = 'POST'
    enctype = ''
    storage = ''
    group_list = []
    icon = PortalContent.getIcon
    _reserved_names = ('confirmation',)
    _fields = []

    def __init__(self, id, title='', fields=[]):
        self._fields = fields
        self.group_list = [Group('Default'),]
        PortalFolder.__init__(self, id, title)
        DefaultDublinCoreImpl.__init__(self, title)

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """ """
        try:
            grp_def = self.group_list[0]
            for g_name, items in self._fields:
                try:
                    grp = self.add_group(g_name)
                    for c_id, c_type, kwargs in items:
                        self.invokeFactory(c_type, c_id, **kwargs)
                        f = self[c_id]
                        grp_def.delField(f)
                        grp.addFields([f,])
                except:
                    traceback.print_exc()
            
            del self._fields
            self.editGroups(0, [], 0, '', 4)
        except:
            traceback.print_exc()
            
        
        try:
            self.invokeFactory('Document', 'confirmation',)
        except: pass
        
        PortalContent.manage_afterAdd(self, item, container)

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, fname, faction, fmethod, fenctype, storage):
        """ edit form """
        self.name = fname
        self.action = faction
        self.enctype = fenctype
        self.storage = storage
        if fmethod in ['POST', 'GET']:
            self.method = fmethod
        self.reindexObject()

    security.declareProtected(ModifyPortalContent, 'editGroups')
    def editGroups(self, group, fieldslist, to_group, title, oper):
        """ edit form groups """
        try:
            if oper==0: # Move fields up
                group = self.group_list[group]
                group.moveFieldsUp(fieldslist)
            elif oper==1: # Move fields down
                group = self.group_list[group]
                group.moveFieldsDown(fieldslist)
            elif oper==2: # Transfer fields
                grp_list = self.Groups()
                group = grp_list[group]
                to_group = grp_list[to_group]
                fs = group.Fields()
                fields = [fs[idx] for idx in fieldslist]
                group.delFields(fieldslist)
                to_group.addFields(fields)
            elif oper == 3: # Rename group
                group = self.group_list[group]
                group.edit(title)
            elif oper == 4: # Remove group
                self.remove_group(group)
        except:
            traceback.print_exc()
            
    security.declareProtected(View, 'Groups')
    def Groups(self):
        """ len of groups """
        return [grp.__of__(self) for grp in self.group_list]
    
    security.declareProtected(View, 'FormName')
    def FormName(self):
        """ """
        return self.name

    security.declareProtected(View, 'FormAction')
    def FormAction(self):
        """ """
        return self.action

    security.declareProtected(View, 'FormMethod')
    def FormMethod(self):
        """ """
        return self.method

    security.declareProtected(View, 'FormEnctype')
    def FormEnctype(self):
        """ """
        return self.enctype

    security.declareProtected(ModifyPortalContent, 'field_added')
    def field_added(self, field):
        """ """
        self.group_list[0].addFields([field])

    security.declareProtected(ModifyPortalContent, 'field_removed')
    def field_removed(self, field):
        """ """
        for grp in self.group_list:
            grp.delField(field)

    security.declareProtected(ModifyPortalContent, 'add_group')
    def add_group(self, group):
        """Add a new group.
        """
        grp = Group(group)
        self.group_list.append(grp)
        self.group_list = self.group_list
        return grp

    security.declareProtected(ModifyPortalContent, 'remove_group')
    def remove_group(self, g_no):
        """ Remove group """
        try:
            group_list = self.group_list
            if len(group_list) > 1:
                old = group_list[g_no]
                del group_list[g_no]
                group_list[0].addFields(old.fields)
                self.group_list = group_list
        except:
            pass

    security.declareProtected(View, 'validate')
    def validate(self, REQUEST):
        """Validate all fields in this form. Stop validating and
        pass up ValidationError if any occurs.
        """
        result = {}
        for field in self.contentValues():
            if isinstance(field, CMFField):
                value = field.validate(REQUEST)
                result[field.getId()] = value
        return result

    security.declareProtected(View, 'validate_to_request')
    def validate_to_request(self, REQUEST):
        """Validation, stop validating as soon as error.
        """
        result = self.validate(REQUEST)
        for key, value in result.items():
            REQUEST.set(key, value)
        return result
    
    security.declareProtected(View, 'validate_all')
    def validate_all(self, REQUEST):
        """Validate all fields in this form, catch any ValidationErrors
        if they occur and raise a FormValidationError in the end if any
        Validation Errors occured.
        """
        result = {}
        errors = []
        for field in self.contentValues():
            try:
                if isinstance(field, CMFField):
                    value = field.validate(REQUEST)
                    # store under id
                    result[field.getId()] = value
            except ValidationError, err:
                errors.append(err)
        if len(errors) > 0:
            raise FormValidationError(errors, result)
        return result

    security.declareProtected(View, 'validate_and_store')
    def validate_and_store(self, REQUEST):
        """ """
        res = self.validate_all(REQUEST)
        storage = self.unrestrictedTraverse(self.storage)
        if (storage.meta_type != 'DataStorage'):
            return

        dict = []
        for grp in self.Groups():
            f_idx = []
            for field in grp.Fields():
                f_idx.append([field.getId(), field.Title()])
            dict.append([grp.title, f_idx])

        res1 = {}
        for key, val in res.items():
            if type(val) == type([]):
                s = ''
                for v in val: s += "%s," % str(v)
                val = '[%s]' % s[:-1]
            res1[key] = val

        storage.append(self, res1, dictionary=dict)
        return res

    
    security.declareProtected(View, 'validate_all_to_request')
    def validate_all_to_request(self, REQUEST):
        """Validation, continue validating all fields, catch errors.
        Everything that could be validated will be added to REQUEST.
        """
        try:
            result = self.validate_all(REQUEST)
        except FormValidationError, e:
            # put whatever result we have in REQUEST
            for key, value in e.result.items():
                REQUEST.set(key, value)
            # reraise exception
            raise
        for key, value in result.items():
            REQUEST.set(key, value)
        return result

    def manage_addFormulatorForm(self, id, title="", fields=[], REQUEST=None):
        """Add form to folder. """
        id = self._setObject(id, CMFForm(id, title, fields))

        if REQUEST is not None:
            return self.folder_contents( # XXX: ick!
                self, REQUEST, portal_status_message="CMF form added")
    


manage_addFormulatorForm = CMFForm.manage_addFormulatorForm
InitializeClass(CMFForm)
